#define _PSTAT64 
#include <sys/param.h> 
#include <sys/pstat.h> 
#include <sys/fstyp.h> 
#include <sys/statvfs.h> 
#include <sys/unistd.h> 
#include <malloc.h> 
#include <stdio.h> 
#include <stdlib.h> 
#include <string.h> 
#include <limits.h> 
#include <mntent.h> 

char *mountpoint(fsid) 
   long fsid;
{ 
    static int init = 0; 
    FILE *mnttab; 
    struct statvfs buf; 
    struct fsrecord { long fsid; char *name; struct fsrecord *next;}; 
    static struct fsrecord *list = NULL; 
    struct fsrecord *record; 
    struct mntent *mnt_entry; 

    if (!init) { 
        init = 1; 
        if ((mnttab = setmntent(MNT_MNTTAB, "r")) != NULL) { 
            while (mnt_entry = getmntent(mnttab)) { 
                statvfs(mnt_entry->mnt_dir, &buf); 
                record = (struct fsrecord *) malloc(sizeof(struct fsrecord)); 
                record->fsid = buf.f_fsid; 
                record->name = strdup(mnt_entry->mnt_dir); 
                record->next = list; 
                list = record; 
            } 
            endmntent(mnttab); 
        } 
    } 
    for (record = list; record != NULL; record = record->next) { 
        if (record->fsid == fsid) return(record->name); 
    } 
    return(""); 
} 

void pstatvm(pid) 
    int pid;
{ 
    struct pst_vm_status pst; 
    int idx, count, i; 
    char fmt[80]; 
    char fstypename[FSTYPSZ]; 

    idx=0; 
    count = pstat_getprocvm(&pst, sizeof(pst), (size_t)pid, idx); 
    while (count > 0) { 
        printf("pid: %d, region: %d\n", pid, idx); 
        printf("pst_space: 0x%llx\n", (long long)pst.pst_space); 
        printf("pst_vaddr64bit_pad: 0x%llx\n", (long long)pst.pst_vaddr64bit_pad); 
        printf("pst_vaddr: 0x%llx\n", (long long)pst.pst_vaddr); 
        printf("pst_length: 0x%llx (%ld)\n",
		(long long)pst.pst_length,
		(long long)pst.pst_length); 
        printf("pst_phys_pages: 0x%llx\n", (long long)pst.pst_phys_pages); 
#ifdef PST_N_PG_SIZES /* only on 11.0 and later */ 
        { 
            printf("pst_pagesize_hint: %lld\n", 
                (long long)pst.pst_pagesize_hint); 
            /* int i; */
            /* Sizes are 4K, 8K, 16K, etc. PA2.0 only uses 4K, 16K, etc. */ 
            printf("page sizes: "); 
            for (i=0; i<PST_N_PG_SIZES; i++) { 
                printf("%lld ", (long long)pst.pst_vps_pgsizes[i]); 
            } 
            printf("\n"); 
        } 
#endif /* PST_N_PG_SIZES */ 
        printf("pst_flags: 0x%llx ", (long long)pst.pst_flags); 
        if (pst.pst_flags & PS_MEMORY_LOCKED) printf("PS_MEMORY_LOCKED "); 
        if (pst.pst_flags & PS_EXECUTABLE) printf("PS_EXECUTABLE "); 
        if (pst.pst_flags & PS_SHARED) printf("PS_SHARED "); 
        if (pst.pst_flags & PS_SHARED_LIBRARY) printf("PS_SHARED_LIBRARY"); 
        printf("\n"); 
        printf("pst_type: 0x%llx ", (long long)pst.pst_type); 
        switch ((long)pst.pst_type) { 
            case PS_NOTUSED: printf("PS_NOTUSED\n"); break; 
            case PS_USER_AREA: printf("PS_USER_AREA\n"); break; 
            case PS_TEXT: printf("PS_TEXT\n"); break; 
            case PS_DATA: printf("PS_DATA\n"); break; 
            case PS_STACK: printf("PS_STACK\n"); break; 
            case PS_SHARED_MEMORY: printf("PS_SHARED_MEMORY\n"); break; 
            case PS_NULLDEREF: printf("PS_NULLDEREF\n"); break; 
            case PS_IO: printf("PS_IO\n"); break; 
            case PS_MMF: printf("PS_MMF\n"); break; 
            case PS_GRAPHICS: printf("PS_GRAPHICS\n"); break; 
        } 
        printf("pst_permission: 0x%llx ", (long long)pst.pst_permission); 
        if (pst.pst_permission & PS_PROT_READ) printf("PS_PROT_READ "); 
        if (pst.pst_permission & PS_PROT_WRITE) printf("PS_PROT_WRITE "); 
        if (pst.pst_permission & PS_PROT_EXECUTE) printf("PS_PROT_EXECUTE "); 
        printf("\n"); 
        printf("pst_id.psf_fsid.psfs_id: 0x%llx ", (long long)pst.pst_id.psf_fsid.psfs_id); 
        printf("%s\n", mountpoint(pst.pst_id.psf_fsid.psfs_id)); 
        sysfs(GETFSTYP, (int) pst.pst_id.psf_fsid.psfs_type, fstypename); 
        printf("pst_id.psf_fsid.psfs_type: %d %s\n", 
            (int) pst.pst_id.psf_fsid.psfs_type, fstypename); 

        printf("pst_id.psf_fileid: %lld\n", (long long)pst.pst_id.psf_fileid); 
        printf("\n"); 

        idx++; 
        count = pstat_getprocvm(&pst, sizeof(pst), (size_t)pid, idx); 
    } 
} 

void pstatvm_all() 
{ 
#define BURST ((size_t)10) 

    struct pst_status pst[BURST]; 
    int i, count; 
    int idx = 0; /* index within the context */ 

    /* loop until count == 0, will occur when all have been returned */ 
    while ((count = pstat_getproc(pst, sizeof(pst[0]), BURST, idx)) > 0) 
    { 
        /* got count (max of BURST) this time. process them */ 
        for (i = 0; i < count; i++) { 
            if (pst[i].pst_pid==0) continue; /* Can't getprocvm real pid 0 */ 
            printf("command: %s, ", pst[i].pst_ucomm); 
#ifdef PST_N_PG_SIZES /* only on 11.0 and later */ 
            printf("+pi: %lld, +pd: %lld, ", 
                (long long) pst[i].pst_text_size, 
                (long long) pst[i].pst_data_size); 
                /* Page sizes are in 4K units. 0 represents default. 
                0 D 
                1 4K 
                4 16K 
                16 4K 
                64 256K 
                256 1M 
                1024 4M 
                4096 16M 
                16384 64M 
                */ 
#endif 
            pstatvm(pst[i].pst_pid); 
        } 

        /* 
         * now go back and do it again, using the next index after 
         * the current 'burst' 
         */ 
        idx = pst[count-1].pst_idx + 1; 
    } 

    if (count == -1) 
        perror("pstat_getproc()"); 

#undef BURST 
} 

main(argc, argv)
    int argc;
    char *argv[];
{ 
    int i; 
    if (argc > 1) { 
        for (i=1; i<argc; i++) 
            pstatvm(atoi(argv[i])); 
    } else { 
        pstatvm_all(); 
    } 
} 


